require "server"
require "utils"

local _logger_name = "Restart_EMS_Sample_Script"
local _extended_retry_frequency = 150
---
--- Install Functions
--- Return true to indicate success and proceed to the next step
--- Return false with a message to indicate failure.
--- The message will be returned and displayed in the history.
---
function Step1 (job)	
	stopEmsService(job)
	startEmsService(job)
	if (job.failed_step ~= nil) then
		if (job.failed_step == "Step1") then
			return false, "Error in Step1"
		end
	end
	return true
end

function stopEmsService (job)
	log.debug(_logger_name, "Stop EMS service...")
	if (tw_utils.isWindows()) then
		log.debug(_logger_name, "Stop Service: ["..job.twems_service_name.."]")
		local result = tw_service.stop(job.twems_service_name, (_extended_retry_frequency * 2))
		if not result then
			log.debug(_logger_name, "Could not stop ems service ["..job.twems_service_name.."]")
			--p_data.rap_port = _ems_port
			return false, "Could not stop ems service ["..job.twems_service_name.."]"
		end
	else
		log.debug(_logger_name, "Actually Stop EMS service...")
		local command = [[/etc/init.d/tw_microServerd stop]]
		local success, result = pcall(os.execute, command)
		if not success then
			--p_data.rap_port = _ems_port
			return false, "Could not stop ems service"
		end
	end
	--Just hang out here a bit to make sure that the process exits (and the port closes)
	tw_utils.psleep(60000)
	return true
end

function startEmsService (job)
	log.debug(_logger_name, "Start EMS service...")
	local success = nil
	
	if (tw_utils.isWindows()) then
		local result = tw_service.start(job.twems_service_name, (_extended_retry_frequency * 2))
		if not result then
			--p_data.rap_port = _ems_port
			return false, "Could not start ems service ["..job.twems_service_name.."]"
		end
	else
		local command = [[/etc/init.d/tw_microServerd start]]
		local success, result = pcall(os.execute, command)
		if not success then
			--p_data.rap_port = _ems_port
			return false, "Could not start ems service"
		end
	end
	log.debug(_logger_name,"RAP_PORT: ".. p_data.rap_port);
	log.debug(_logger_name,"EMS_PORT: ".. (_ems_port or ""));
	--use the original ems port again
	--p_data.rap_port = _ems_port

	return waitForEMSToStart()
end

function waitForEMSToStart()
	local success = false
	local code = nil
	local result = nil
	
	local startTime = os.time()
	
	log.debug(_logger_name, "Wait for EMS to go online")
	while not success do
		success, code, result = pcall(thingworx.server.get, "/properties/status")
		log.debug(_logger_name, "Get status from EMS...success:[%s], code:[%s]", tostring(success), tostring(code))
		if (not success) or (code ~= 200) or (result ~= "online") then
			log.debug(_logger_name, "EMS is offline, retrying...")
			--wait for 5 seconds
			tw_utils.psleep(5000)
			local now = os.time()
			if ((now - startTime) > _extended_retry_frequency) then
				--p_data.rap_port = _ems_port
				return false, "The EMS could not connect."
			end
		end
	end
	log.debug(_logger_name, "SUCCESS_LOG: Get status from EMS...success:[%s], code:[%s], result:[%s]", tostring(success), tostring(code), tostring(result))
	return true
end

---
--- Rollback Functions
--- The rollback functions are optional
--- The rollback functions must be named rollback_<StepFunctionName>
--- for the rollback process to work.
--- The steps of the rollback are performed in reverse order of the install steps.
---
function rollback_Step1(job)
	--do rollback functionality here
end

steps = {
	"Step1"
}
